@@ -4,16 +4,9 @@ import time
 import asyncclick as click
 import datetime
 
-from distkv.util import (
-    PathLongener,
-    MsgReader,
-    PathShortener,
-    NotGiven,
-    yprint,
-    data_get,
-    path_eval,
-)
+from distkv.util import PathLongener, MsgReader, NotGiven, yprint, data_get, P
 from distkv.client import StreamedRequest
+from distkv.command import node_attr
 
 import logging
 
@@ -37,28 +30,15 @@ async def cli():
     "for values. Default: return as list",
 )
 @click.option(
-    "-m",
-    "--maxdepth",
-    type=int,
-    default=None,
-    help="Limit recursion depth. Default: whole tree",
+    "-m", "--maxdepth", type=int, default=None, help="Limit recursion depth. Default: whole tree"
 )
 @click.option(
-    "-M",
-    "--mindepth",
-    type=int,
-    default=None,
-    help="Starting depth. Default: whole tree",
+    "-M", "--mindepth", type=int, default=None, help="Starting depth. Default: whole tree"
 )
 @click.option("-r", "--recursive", is_flag=True, help="Read a complete subtree")
 @click.option("-e", "--empty", is_flag=True, help="Include empty nodes")
-@click.option(
-    "-V", "--eval-path", type=int, multiple=True, help="Eval this path element"
-)
-@click.option(
-    "-R", "--raw", is_flag=True, help="Print string values without quotes etc."
-)
-@click.argument("path", nargs=-1)
+@click.option("-R", "--raw", is_flag=True, help="Print string values without quotes etc.")
+@click.argument("path", nargs=1)
 @click.pass_obj
 async def get(obj, path, **k):
     """
@@ -69,7 +49,8 @@ async def get(obj, path, **k):
     for incremental output.
     """
 
-    await data_get(obj, *path, **k)
+    path = P(path)
+    await data_get(obj, path, **k)
 
 
 @cli.command("list")
@@ -88,13 +69,9 @@ async def get(obj, path, **k):
     help="Limit recursion depth. Default: 1 (single layer).",
 )
 @click.option(
-    "-M",
-    "--mindepth",
-    type=int,
-    default=1,
-    help="Starting depth. Default: 1 (single layer).",
+    "-M", "--mindepth", type=int, default=1, help="Starting depth. Default: 1 (single layer)."
 )
-@click.argument("path", nargs=-1)
+@click.argument("path", nargs=1)
 @click.pass_obj
 async def list_(obj, path, **k):
     """
@@ -107,75 +84,72 @@ async def list_(obj, path, **k):
     for incremental output.
     """
 
+    path = P(path)
     k["recursive"] = True
     k["raw"] = True
     k["empty"] = True
-    await data_get(obj, *path, **k)
+    await data_get(obj, path, **k)
 
 
 @cli.command("set", short_help="Add or update an entry")
 @click.option("-v", "--value", help="The value to store. Mandatory.")
-@click.option(
-    "-e", "--eval", "eval_", is_flag=True, help="The value shall be evaluated."
-)
-@click.option(
-    "-p", "--prev", default=NotGiven, help="Previous value. Deprecated; use 'last'"
-)
+@click.option("-e", "--eval", "eval_", is_flag=True, help="The value shall be evaluated.")
+@click.option("-P", "--path", "path_", is_flag=True, help="The value is a path.")
 @click.option("-l", "--last", nargs=2, help="Previous change entry (node serial)")
 @click.option("-n", "--new", is_flag=True, help="This is a new entry.")
-@click.option(
-    "-V", "--eval-path", type=int, multiple=True, help="Eval this path element"
-)
-@click.argument("path", nargs=-1)
+@click.option("-a", "--attr", default=None, help="Modify attribute.")
+@click.argument("path", nargs=1)
 @click.pass_obj
-async def set_(obj, path, eval_path, value, eval_, prev, last, new):
+async def set_(obj, path, value, eval_, last, new, path_, attr):
     """
     Store a value at some DistKV position.
 
-    If you update a value, you really should use "--last" (preferred) or
-    "--prev" (if you must) to ensure that no other change arrived.
+    If you update a value, you should use "--last"
+    to ensure that no other change arrived.
 
     When adding a new entry, use "--new" to ensure that you don't
     accidentally overwrite something.
     """
+    path = P(path)
+    if attr is not None:
+        attr = P(attr)
     if eval_:
-        value = eval(value)  # pylint: disable=eval-used
+        if path_:
+            raise click.UsageError("'eval' and 'path' are mutually exclusive")
+        if value == "-":
+            value = NotGiven
+        else:
+            value = eval(value)  # pylint: disable=eval-used
+            eval_ = False
+    elif path_:
+        value = P(value)
     args = {}
     if new:
-        if prev is not NotGiven or last:
-            raise click.UsageError("'new' and 'prev'/'last' are mutually exclusive")
+        if last:
+            raise click.UsageError("'new' and 'last' are mutually exclusive")
         args["chain"] = None
     else:
-        if prev is not NotGiven:
-            if eval_:
-                prev = eval(prev)  # pylint: disable=eval-used
-            args["prev"] = prev
         if last:
             args["chain"] = {"node": last[0], "tick": int(last[1])}
 
-    res = await obj.client.set(
-        *path_eval(path, eval_path), value=value, nchain=obj.meta, **args
-    )
+    if attr is not None:
+        res = await node_attr(obj, path, attr, value, eval_=eval_, **args)
+    else:
+        res = await obj.client.set(path, value=value, nchain=obj.meta, **args)
+
     if obj.meta:
         yprint(res, stream=obj.stdout)
 
 
 @cli.command(short_help="Delete an entry / subtree")
-@click.argument("path", nargs=-1)
-@click.option(
-    "-p", "--prev", default=NotGiven, help="Previous value. Deprecated; use 'last'"
-)
+@click.argument("path", nargs=1)
+@click.option("-p", "--prev", default=NotGiven, help="Previous value. Deprecated; use 'last'")
 @click.option("-l", "--last", nargs=2, help="Previous change entry (node serial)")
 @click.option("-r", "--recursive", is_flag=True, help="Delete a complete subtree")
 @click.option("--internal", is_flag=True, help="Affect the internal tree. DANGER.")
-@click.option(
-    "-V", "--eval-path", type=int, multiple=True, help="Eval this path element"
-)
-@click.option(
-    "-e", "--eval", "eval_", is_flag=True, help="The previous value shall be evaluated."
-)
+@click.option("-e", "--eval", "eval_", is_flag=True, help="The previous value shall be evaluated.")
 @click.pass_obj
-async def delete(obj, path, eval_path, prev, last, recursive, eval_, internal):
+async def delete(obj, path, prev, last, recursive, eval_, internal):
     """
     Delete an entry, or a whole subtree.
 
@@ -188,6 +162,7 @@ async def delete(obj, path, eval_path, prev, last, recursive, eval_, internal):
 
     The root entry cannot be deleted.
     """
+    path = P(path)
     args = {}
     if eval_ and prev is NotGiven:
         raise click.UsageError("You need to add a value that can be evaluated")
@@ -205,12 +180,8 @@ async def delete(obj, path, eval_path, prev, last, recursive, eval_, internal):
             args["chain"] = {"node": last[0], "tick": int(last[1])}
 
     res = await obj.client._request(
-        action="delete_tree"
-        if recursive
-        else "delete_internal"
-        if internal
-        else "delete_value",
-        path=tuple(path_eval(path, eval_path)),
+        action="delete_tree" if recursive else "delete_internal" if internal else "delete_value",
+        path=path,
         nchain=obj.meta,
         **args
     )
@@ -227,25 +198,21 @@ async def delete(obj, path, eval_path, prev, last, recursive, eval_, internal):
 
 @cli.command()
 @click.option("-s", "--state", is_flag=True, help="Also get the current state.")
-@click.option(
-    "-V", "--eval-path", type=int, multiple=True, help="Eval this path element"
-)
-@click.argument("path", nargs=-1)
+@click.argument("path", nargs=1)
 @click.pass_obj
-async def watch(obj, path, eval_path, state):
+async def watch(obj, path, state):
     """Watch a DistKV subtree"""
-    flushing = not state
-    path = tuple(path_eval(path, eval_path))
 
-    async with obj.client.watch(*path, nchain=obj.meta, fetch=state) as res:
-        pl = PathLongener(path)
+    flushing = not state
+    path = P(path)
+
+    async with obj.client.watch(path, nchain=obj.meta, fetch=state) as res:
         async for r in res:
-            pl(r)
             if not flushing and r.get("state", "") == "uptodate":
                 flushing = True
             del r["seq"]
             r["time"] = time.monotonic()
-            r["date"] = datetime.datetime.now().strftime("%S-%m-%d %H:%M:%S")
+            r["date"] = datetime.datetime.now().strftime("%Y-%m-%d %H:%M:%S")
             yprint(r, stream=obj.stdout)
             print("---", file=obj.stdout)
             if flushing:
@@ -253,27 +220,12 @@ async def watch(obj, path, eval_path, state):
 
 
 @cli.command()
-@click.option("-l", "--local", is_flag=True, help="Load locally, don't broadcast")
-@click.option("-f", "--force", is_flag=True, help="Overwrite existing values")
-@click.option("-i", "--infile", type=click.File("rb"), help="File to read (msgpack).")
-@click.option(
-    "-V", "--eval-path", type=int, multiple=True, help="Eval this path element"
-)
-@click.argument("path", nargs=-1)
+@click.option("-i", "--infile", type=click.Path(), help="File to read (msgpack).")
+@click.argument("path", nargs=1)
 @click.pass_obj
-async def update(obj, path, eval_path, infile, local, force):
+async def update(obj, path, infile):
     """Send a list of updates to a DistKV subtree"""
-    if local and force:
-        raise click.UsageError("'local' and 'force' are mutually exclusive")
-
-    path = tuple(path_eval(path, eval_path))
-    ps = PathShortener(path)
-    async with MsgReader(stream=infile) as reader:
-        with obj.client._stream(
-            action="update", path=path, force=force, local=local
-        ) as sender:
-            async for r in reader:
-                ps(r)
-                await sender.send(r)
-
-    print(sender.result)
+    path = P(path)
+    async with MsgReader(path=infile) as reader:
+        async for msg in reader:
+            await obj.client.set(*path, *msg.path, value=msg.value)
